package chat

import zgs.httpd._

object ChatServer extends HServer with App {
  
  protected def ports = Set(8910)
  protected def apps = List(chatApp, staticApp)
  
  override def stop : Unit = {
    super.stop
    ChatApp.stop
  }
  
  private object chatApp extends HApp {
    
    override def buffered = true

    object sendLet extends HLet {
      def act(tk: HTalk) = {
        for { nic  <- tk.req.param("nic"); text <- tk.req.param("text") } yield ChatApp.acceptMsg(nic, text)
        tk.setContentType(HMime.txt).write("Ok")
      }
    }
      
    object pollLet extends HLet {
      
      override def talkExecutor = Some(ChatApp) // this is what we are testing
      
      def act(tk: HTalk) = {
        tk.setContentType(HMime.xml)
        def after = tk.req.asLong("after").getOrElse(Long.MaxValue) // without 'after' <messages></messages> will be returned
        tk.write(<messages>{ChatApp.getMessages(after).map(_.toXml)}</messages> toString) // may be <messages></messages> for ping
      }
    }

    import HTree._
    private val tree = 
      "" += (
        "send" ! sendLet,
        "poll" ! pollLet
      )
    def resolve(req: HReqData) = tree.resolve(req.uriPath)
  }

  private object staticApp extends HApp {
    
    override def buffered : Boolean  = true // ResourceLet needs buffered or chunked be set
    
    object staticLet extends let.ResourceLet {
      protected def dirRoot          = "static"
      override protected def uriRoot = ""
      override protected def indexes = List("index.html")
      override protected def bufSize = ChatServer.bufferSize
    }
    
    def resolve(req: HReqData) = Some(staticLet) // generates 404
  }
  
  // why do not start all these mechanics?
  start
}
